/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "DocumentType.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDocumentType.h"
#include "gdom/GdomNode.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDocumentType.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateIntGdomDocumentType.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMDocumentType::GDOMDocumentType(PassRefPtr<WebCore::DocumentType> impl)
    : GDOMNode(impl)
{
}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "DocumentType.h"
#include <gdom/GdomDocumentType.h>
#include <gdom/PrivateGdomDocumentType.h>
#include <gdom/PrivateIntGdomDocumentType.h>
#include <gdom/GdomDocumentTypeprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomDocumentType, gdom_document_type, GDOM_TYPE_NODE);

/*
struct _GdomDocumentTypePrivate {
    class WebCore::DocumentType *item;
};

#define GDOM_DOCUMENT_TYPE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_DOCUMENT_TYPE, GdomDocumentTypePrivate))
*/
G_END_DECLS
WebCore::DocumentType *
gdom_document_type_get_private(GdomDocumentType *request)
{
    GDOMDocumentType *res;
    res = static_cast<GDOMDocumentType*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::DocumentType*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_NAME,
    PROP_ENTITIES,
    PROP_NOTATIONS,
    PROP_PUBLIC_ID,
    PROP_SYSTEM_ID,
    PROP_INTERNAL_SUBSET,
};


static void gdom_document_type_finalize(GObject* object)
{
    /*GdomDocumentType* request = GDOM_DOCUMENT_TYPE(object);*/
    /*GdomDocumentTypePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_NODE));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_document_type_parent_class)->finalize(object);
}

static void gdom_document_type__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomDocumentType *self = GDOM_DOCUMENT_TYPE(object);
    WebCore::DocumentType * gself = (gdom_document_type_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_document_type__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomDocumentType *self = GDOM_DOCUMENT_TYPE(object);
    WebCore::DocumentType * gself = (gdom_document_type_get_private(self));
    switch (prop_id) {
    case PROP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->name()));
         break;
    }
    case PROP_ENTITIES:
    {
         PassRefPtr<WebCore::NamedNodeMap> ptr = gself->entities();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_NOTATIONS:
    {
         PassRefPtr<WebCore::NamedNodeMap> ptr = gself->notations();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PUBLIC_ID:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->publicId()));
         break;
    }
    case PROP_SYSTEM_ID:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->systemId()));
         break;
    }
    case PROP_INTERNAL_SUBSET:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->internalSubset()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_document_type_class_init(GdomDocumentTypeClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_document_type_finalize;
    g_klass->set_property = gdom_document_type__set_property_fn;
    g_klass->get_property = gdom_document_type__get_property_fn;

    g_object_class_install_property (g_klass, PROP_NAME,
            g_param_spec_string ("name", /* name */
                "document_type_name", /* short description */
                "read-only  gchar * DocumentType.name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ENTITIES,
            g_param_spec_object ("entities", /* name */
                "document_type_entities", /* short description */
                "read-only  GdomNamedNodeMap * DocumentType.entities", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NAMED_NODE_MAP, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NOTATIONS,
            g_param_spec_object ("notations", /* name */
                "document_type_notations", /* short description */
                "read-only  GdomNamedNodeMap * DocumentType.notations", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NAMED_NODE_MAP, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PUBLIC_ID,
            g_param_spec_string ("public_id", /* name */
                "document_type_public_id", /* short description */
                "read-only  gchar * DocumentType.public_id", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SYSTEM_ID,
            g_param_spec_string ("system_id", /* name */
                "document_type_system_id", /* short description */
                "read-only  gchar * DocumentType.system_id", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_INTERNAL_SUBSET,
            g_param_spec_string ("internal_subset", /* name */
                "document_type_internal_subset", /* short description */
                "read-only  gchar * DocumentType.internal_subset", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomDocumentTypePrivate));*/
}

static void gdom_document_type_init(GdomDocumentType* request)
{
    /*GdomDocumentTypePrivate* priv = GDOM_DOCUMENT_TYPE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

